import objectdraw.*;
import java.awt.*;

/* $Id: NibbleField.java 1591 2011-04-06 03:29:47Z terescoj $ */

/**
 * The 2D grid that the snake roams around on
 *
 * @author A. Danyluk, B. Lerner April 2000
 * @author Modifed by Tom Murtagh, Nov 2001
 * @author Modified by Jim Teresco, April 2011
 */

public class NibbleField {
    // Size of a single cell in the field
    private static final int CELL_SIZE = 14;
    
    // Minimum amount of blank space between field and canvas
    private static final int FIELD_INSET = 10;
    
    // Colors to use when drawing food or pieces of the snake
    private static final Color FOOD_COLOR = Color.red;
    private static final Color SNAKE_COLOR = Color.green;
    
    // Dimensions of the field measured in cells
    private int fieldWidth;
    private int fieldHeight;
    
    // The grid contains rectangles.  A null entry indicates an empty
    // slot.  If a rectangle is present, its color will indicate to
    // the snake what it is.
    private FilledRect[][] theField;
    
    // upper left corner of the field on the display
    private int cornerX, cornerY;
    
    // the drawing canvas
    private DrawingCanvas canvas;
    
    // Current location of the food
    private Position foodPos;
    
    // Random number generators used to place food.
    RandomIntGenerator rowGenerator, colGenerator;
    
    /**
     * Construct a new field
     * 
     * @param canvas the DrawingCanvas on which to display the grid
     */
    public NibbleField(DrawingCanvas canvas) {
        this.canvas = canvas;
        
        cornerX = FIELD_INSET;
        cornerY = FIELD_INSET;
        
        fieldWidth =(int) ( canvas.getWidth() - 2*FIELD_INSET)/CELL_SIZE;
        fieldHeight =(int) ( canvas.getHeight() - 2*FIELD_INSET)/CELL_SIZE;
        
        // Create the array use to record contents of the field.
        // note that it will contain all null values when
        // constructed, so we do not need to initialize
        theField = new FilledRect[fieldHeight][fieldWidth];
        
        //draws the borders of the field
        new FramedRect(cornerX - 1, cornerY -1 , CELL_SIZE*fieldWidth+1,
                       CELL_SIZE*fieldHeight+1, canvas);
        
        // Create random number generators used to position food
        rowGenerator = new RandomIntGenerator (0, fieldHeight-1);
        colGenerator = new RandomIntGenerator (0, fieldWidth-1);
        
        
        //draws the initial food to "eat"
        
        placeFood();
        
    }
    
    
    /***********************************************************************/
    /*     P U B L I C   M E T H O D S                                     */
    /***********************************************************************/
    
    /**
     * return true if the specified position in the field holds food
     *
     * @param pos the position to check
     *
     * @return true if the specified position in the field holds food
     */
    public boolean containsFood (Position pos) {
        return pos.equals(foodPos);
    }
    
    /**
     * return true if the specified position in the field holds part
     * of the snake
     *
     * @param pos the position to check
     *
     * @return true if the specified position in the field holds snake
     */
    public boolean containsSnake (Position pos) {
        return isOccupied(pos) && ! containsFood(pos);
    }
    
    /**
     * returns whether a position is out of the bounds of the field
     *
     * @param pos the position to check
     *
     * @return true if the specified position is out of bounds
     */
    public boolean outOfBounds(Position pos) {
        
        return pos.getRow() < 0 || pos.getRow() >= fieldHeight ||
               pos.getCol() < 0 || pos.getCol() >= fieldWidth;
        
    }
    
    /**
     * return the position of the center of the field
     *
     * @return the position of the center of the field
     */
    public Position getCenter() {
        return new Position(fieldHeight/2, fieldWidth/2);
    }

    /**
     * Move the food to a new location on the field
     */
    public void consumeFood() {
        Position oldPos = foodPos;
        placeFood();
        removeContents(oldPos);
    }
    
    /**
     * Display a piece of the snake as a rectangle at a specified position
     *
     * @param pos row and column of the cell to contain the rectangle
     */
    public void addItem (Position pos) {
        if (!outOfBounds(pos)) {
            if (isOccupied(pos)) {
                System.out.println ("WARNING:  Adding item to " + pos + " but it is not empty!");
                Thread.dumpStack();
            }
            showContents(pos, SNAKE_COLOR);
        }
    }
    
    /**
     * Remove a piece of the snake from the field.  Complains if the
     * position is already empty.
     *
     * @param pos row and column of the cell to remove from
     */
    public void removeItem (Position pos) {
        if (!isOccupied(pos)) {
            System.out.println ("WARNING:  Removing item from " + pos + " but it is empty!");
            Thread.dumpStack();
        }
        removeContents(pos);
    }
    
    /***********************************************************************/
    /*     P R I V A T E   M E T H O D S                                   */
    /***********************************************************************/
    
    // Place a new piece of food on the screen somewhere away from snake
    private void placeFood() {
        
        // Randomly generate locations until we find an empty one.
        do {
            foodPos = new Position (rowGenerator.nextValue(), colGenerator.nextValue());
        } while (isOccupied (foodPos) );
        
        showContents( foodPos, FOOD_COLOR );
        
    }
    
    // returns whether a particular field position is occupied
    // Parameters:
    //    pos - row and column of the cell to check
    private boolean isOccupied(Position pos)
    {
        int row = pos.getRow();
        int col = pos.getCol();
        
        return (theField[row][col] != null);
    }
    
    
    // Remove the rectangle representing the food or a piece of the snake
    // Parameters:
    //    pos - row and column of the cell to remove the item from
    private void removeContents( Position pos ) {
        int row = pos.getRow();
        int col = pos.getCol();
        
        theField[row][col].removeFromCanvas();
        theField[row][col] = null;
    }
    
    
    // Add a rectangle representing the food or a piece of the snake
    // Parameters:
    //    pos - row and column of the cell in which to place the item
    private void showContents( Position pos, Color hue ) {
        int row = pos.getRow();
        int col = pos.getCol();
        
        theField[row][col] = new FilledRect(cornerX+CELL_SIZE*col,
                                            cornerY+CELL_SIZE*row,
                                            CELL_SIZE, CELL_SIZE,
                                            canvas);
        theField[row][col].setColor( hue );
    }
    
}
